using WingServerCommon.Config.Parameters;

namespace WingServerCommon.Config
{
    /// <summary>
    /// 环境配置帮助类
    /// </summary>
    public class EnvironmentConfigs
    {
        public static General General { get; } = new General();
        public static Rtmp Rtmp { get; } = new Rtmp();
        public static Rtc Rtc { get; } = new Rtc();
        public static Storage Storage { get; } = new Storage();
    }

    public class BaseEnvironment
    {
        protected readonly string _name;

        public BaseEnvironment()
        {
            _name = GetType().Name;
        }
    }


    /// <summary>
    /// General
    /// </summary>
    public class General : BaseEnvironment
    {
        public General() : base()
        {
        }

        public string LiveProtocol
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "LiveProtocol").Value;
            }
        }

        /// <summary>
        /// General read live config
        /// </summary>
        public T LiveProtocolDetail<T>()
        {
            var t = default(T);
            if (LiveProtocol.ToLower() == "rtmp")
            {
                t = (T)Enum.Parse(typeof(T), EnvironmentConfigs.Rtmp.RTMPType);
            }
            else
            {
                t = (T)Enum.Parse(typeof(T), EnvironmentConfigs.Rtc.RTCType);
            }
            return t;
        }
    }

    /// <summary>
    /// RTMP
    /// </summary>
    public class Rtmp : BaseEnvironment
    {
        public Rtmp() : base()
        {
        }

        public string RTMPType
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTMPType").Value;
            }
        }

        public string RTMPAppId
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTMPAppId").Value;
            }
        }

        public string RTMPAppKey
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTMPAppKey").Value;
            }
        }

        public string RTMPPlayUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTMPPlayUrl").Value;
            }
        }

        public string RTMPPushUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTMPPushUrl").Value;
            }
        }
    }

    /// <summary>
    /// RTC
    /// </summary>
    public class Rtc : BaseEnvironment
    {
        public Rtc() : base()
        {
        }

        public string RTCType
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTCType").Value;
            }
        }

        public string AppId
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "AppId").Value;
            }
        }

        public string RTCAppKey
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTCAppKey").Value;
            }
        }

        public int RTCBizid
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<IntParameter>(_name, "RTCBizid").Value;
            }
        }

        public string RTCPlayUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTCPlayUrl").Value;
            }
        }

        public string RTCPushUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTCPushUrl").Value;
            }
        }

        public string RTCStartingRoomId
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "RTCStartingRoomId").Value;
            }
        }
    }

    /// <summary>
    /// Storage
    /// </summary>
    public class Storage : BaseEnvironment
    {
        public Storage() : base()
        {
        }

        public int AutoSliceSizeForUpload
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<IntParameter>(_name, "AutoSliceSizeForUpload").Value;
            }
        }

        public int ManualDivisionForUpload
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<IntParameter>(_name, "ManualDivisionForUpload").Value;
            }
        }

        public string OtherNodes
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "OtherNodes").Value;
            }
        }

        public string ServerHost
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "ServerHost").Value;
            }
        }

        public string StorageServer
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "StorageServer").Value;
            }
        }
        public string AppId
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "AppId").Value;
            }
        }

        public string Sercret
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "Sercret").Value;
            }
        }

        public string CFSServerUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "CFSServerUrl").Value;
            }
        }

        public string CDNServerUrl
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "CDNServerUrl").Value;
            }
        }
        public string VCSStorageDisk
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "VCSStorageDisk").Value;
            }
        }
    }

    /// <summary>
    /// Gateway
    /// </summary>
    public class Gateway : BaseEnvironment
    {
        public Gateway() : base()
        {
        }

        public string Host
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "Host").Value;
            }
        }

        public string Domains
        {
            get
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(_name, "Domains").Value;
            }
        }
    }
    
}

