using System;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.OpLog;
namespace WingServerCommon.Interfaces.Cache
{
    public interface IConsultationRecordManager : IBaseCacheManager<CacheConsultationRecordDTO>
    {

    }

    public class ConsultationRecordManager : CacheManager<CacheConsultationRecordDTO>, IConsultationRecordManager
    {
        public ConsultationRecordManager() : base()
        {
        }
    }

    /// <summary>
    /// 会诊信息
    /// </summary>
    public class CacheConsultationRecordDTO : ICacheObject
    {
        /// <summary>
        /// 会诊编码
        /// </summary>
        public string Code { get; set; } = string.Empty;

        /// <summary>
        /// 申请机构编码
        /// </summary>
        public string ApplyOrganizationCode { get; set; }

        /// <summary>
        /// 申请科室编码
        /// </summary>
        public string ApplyDepartmentCode { get; set; }

        /// <summary>
        /// 申请医生编码
        /// </summary>
        public string ApplyUserCode { get; set; }

        /// <summary>
        /// 会诊机构编码
        /// </summary>
        public string ExpertOrganizationCode { get; set; }

        /// <summary>
        /// 会诊科室编码
        /// </summary>
        public string ExpertDepartmentCode { get; set; }

        /// <summary>
        /// 会诊专家编码
        /// </summary>
        public string ExpertUserCode { get; set; }

        /// <summary>
        /// 设备编码
        /// </summary>
        public string DeviceCode { get; set; }

        /// <summary>
        /// 病人编码
        /// </summary>
        public string PatientCode { get; set; }

        /// <summary>
        /// 会诊开始时间
        /// </summary>
        /// <value>0</value>
        public DateTime ConsultationTime { get; set; }

        /// <summary>
        /// 会诊截止时间
        /// </summary>
        /// <value></value>
        public DateTime ConsultationTimeEnd { get; set; }

        /// <summary>
        /// 会诊状态
        /// </summary>
        /// <value>0</value>
        public int ConsultationStatus { get; set; }

        /// <summary>
        /// 会诊参与方对象列表
        /// </summary>
        /// <value></value>
        public List<CacheConsultationMemberDO> ConsultationMembers { get; set; }

        /// <summary>
        /// 发起人编码
        /// </summary>
        /// <value></value>
        public string InitiatorCode { get; set; }

        /// <summary>
        /// 是否急诊标识
        /// </summary>
        /// <value></value>
        public bool IsEmergency { get; set; }

        /// <summary>
        /// 急诊code
        /// </summary>
        /// <value></value>
        public string EmergencyCode { get; set; }

        /// <summary>
        /// 急诊实时状态
        /// </summary>
        /// <value></value>
        public int EmergencyStatus { get; set; }
    }

    /// <summary>
    /// 参与成员对象
    /// </summary>
    public class CacheConsultationMemberDO
    {
        /// <summary>
        /// 参与人员编码
        /// </summary>
        /// <value></value>
        public string MemberCode { get; set; }

        /// <summary>
        /// 参与人员机构编码
        /// </summary>
        /// <value></value>
        public string MemberOrganizationCode { get; set; }

        /// <summary>
        /// 参与人员科室编码
        /// </summary>
        /// <value></value>
        public string MemberDepartmentCode { get; set; }

        /// <summary>
        /// 参与人员状态
        /// </summary>
        /// <value></value>
        public int MemberStatus { get; set; }

        /// <summary>
        /// 参与人员操作时间
        /// </summary>
        /// <value></value>
        public DateTime MemberOperateTime { get; set; }

        /// <summary>
        /// 拒绝原因
        /// </summary>
        /// <value></value>
        public string RefusalCause { get; set; }
    }
}