using System;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.OpLog;
namespace WingServerCommon.Interfaces.Cache
{
    /// <summary>
    /// 课程缓存
    /// </summary>
    public interface ICourseInfosManager : IBaseCacheManager<CacheCourseInfoDTO>
    {

    }

    /// <summary>
    /// 课程缓存
    /// </summary>
    public class CourseInfosManager : CacheManager<CacheCourseInfoDTO>, ICourseInfosManager
    {
        public CourseInfosManager() : base()
        {
        }
    }

    /// <summary>
    /// 课程信息
    /// </summary>
    public class CacheCourseInfoDTO : ICacheObject
    {
        /// <summary>
        /// 课程编码
        /// </summary>
        public string Code { get; set; } = string.Empty;

        /// <summary>
        /// 课程名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 教师Code
        /// </summary>
        public string TeacherCode { get; set; }

        /// <summary>
        ///教师名称
        /// </summary>
        public string TeacherName { get; set; }

        /// <summary>
        /// 课程时长【单位分钟】
        /// </summary>
        public int Duration { get; set; }

        /// <summary>
        /// 课程状态
        /// </summary>
        public int Status { get; set; }

        /// <summary>
        /// 课程价格
        /// </summary>
        public decimal Price { get; set; }

        /// <summary>
        /// 面向类型
        /// </summary>
        public int AudienceType { get; set; }

        /// <summary>
        /// 封面图，课程海报
        /// </summary>
        public string Poster { get; set; }

        /// <summary>
        /// 创建机构code
        /// </summary>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 课程预计开始时间
        /// </summary>
        public DateTime StartTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 课程结束签到时间
        /// </summary>
        public DateTime Deadline { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 实际开始时间
        /// </summary>
        public DateTime ActualStartTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 实际结束时间
        /// </summary>
        public DateTime ActualEndTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 病例标签
        /// </summary>
        public IList<string> CaseLabelCodes { get; set; }

        /// <summary>
        /// 课程分类
        /// </summary>
        public int CourseType { get; set; }

        /// <summary>
        /// 报名学生列表
        /// </summary>
        public IList<CacheStudentInfoDTO> Students { get; set; } = new List<CacheStudentInfoDTO>();

        /// <summary>
        /// 助教列表
        /// </summary>
        public IList<CacheStudentInfoDTO> Assistants { get; set; } = new List<CacheStudentInfoDTO>();

        /// <summary>
        /// 参与互动专家
        /// </summary>
        public IList<CacheStudentInfoDTO> Experts { get; set; } = new List<CacheStudentInfoDTO>();

        /// <summary>
        /// 教培视频Code列表
        /// </summary>
        public IList<string> CourseVideoCodes { get; set; }
    }

    /// <summary>
    /// 参与成员对象
    /// </summary>
    public class CacheStudentInfoDTO
    {
         /// <summary>
        /// 用户code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 是否支付：是-支付；否-未支付
        /// </summary>
        public bool IsPay { get; set; }

        /// <summary>
        /// 学生对于课程的状态
        /// </summary>
        ///  <value>0</value>
        public int SignCourseStatus { get; set; }
    }
}