using System;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.OpLog;
namespace WingServerCommon.Interfaces.Cache
{
    public interface IDeviceInfosManager : IBaseCacheManager<CacheDeviceDTO>
    {

    }

    public class DeviceInfosManager : CacheManager<CacheDeviceDTO>, IDeviceInfosManager
    {
        public DeviceInfosManager() : base()
        {
        }
    }

    public class CacheDeviceDTO : ICacheObject
    {
        /// <summary>
        /// 设备编码
        /// </summary>
        /// <value>test0122</value>
        public string Code { get; set; }

        /// <summary>
        /// 设备序列号
        /// </summary>
        /// <value>test010122</value>
        public string SerialNumber { get; set; }

        /// <summary>
        /// 设备密码
        /// </summary>
        /// <value>ABC123</value>
        public string Password { get; set; }

        /// <summary>
        /// 设备名称
        /// </summary>
        /// <value>测试设备0122</value>
        public string Name { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        /// <value>测试Description22</value>
        public string Description { get; set; }

        /// <summary>
        /// 设备型号
        /// </summary>
        /// <value>M001</value>
        public string DeviceModel { get; set; }

        /// <summary>
        /// 设备类型
        /// </summary>
        /// <value>X00001</value>
        public string DeviceType { get; set; }

        /// <summary>
        /// 设备头图
        /// </summary>
        /// <value>http://192.168.6.117:9001/Storage/B1154C6500E4F174249224B41284498E_7_0_0_0.PNG</value>
        public string HeadPicUrl { get; set; }

        /// <summary>
        /// 设备软件版本
        /// </summary>
        /// <value>1.11.0</value>
        public string DeviceSoftwareVersion { get; set; }

        /// <summary>
        /// 设备SDK版本
        /// </summary>
        /// <value>1.11.1</value>
        public string SDKSoftwareVersion { get; set; }

        /// <summary>
        /// 设备所属组织
        /// </summary>
        /// <value>Organization_2022012520473430DnW3</value>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 组织下属科室
        /// </summary>
        /// <value>Organization_2022012520473430DnW32</value>
        public string DepartmentCode { get; set; }

        /// <summary>
        /// 超声机动态唯一码
        /// </summary>
        /// <value>test01010122</value>
        public string ShortCode { get; set; }

        /// <summary>
        /// 是否设备自动分享
        /// </summary>
        /// <value>true</value>
        public bool IsAutoShared { get; set; }

        /// <summary>
        /// 最近登录时间
        /// </summary>
        /// <value>2022-02-28T09:21:38.861Z</value>
        public DateTime LastLoginTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 系统版本
        /// </summary>
        /// <value>2.2.27.5</value>
        public string SystemVersion { get; set; } = string.Empty;

        /// <summary>
        /// CPU型号
        /// </summary>
        /// <value>I5-10400</value>
        public string CPUModel { get; set; } = string.Empty;

        /// <summary>
        /// 系统语言
        /// </summary>
        /// <value>Chinese</value>
        public string SystemLanguage { get; set; } = string.Empty;

        /// <summary>
        /// AI应用集合
        /// </summary>
        public List<string> DiagnosisModules { get; set; } = new();

        /// <summary>
        /// 报告转发配置集合
        /// </summary>
        public List<string> ReportPosterCodes { get; set; } = new();

        /// <summary>
        /// 是否把多路视屏合流
        /// </summary>
        /// <value></value>
        public bool MergedChannel { get; set; }

        /// <summary>
        /// 合流的画面宽度
        /// </summary>
        /// <value></value>
        public int MergedVideoOutputWidth { get; set; }

        /// <summary>
        /// 合流的画面高度
        /// </summary>
        /// <value></value>
        public int MergedVideoOutputHeight { get; set; }

        /// <summary>
        /// 下载地址模式
        /// </summary>
        /// <value></value>
        public int DownloadModeSetting { get; set; }

        /// <summary>
        /// 视屏设备画面信息
        /// </summary>
        /// <value></value>
        public IList<CacheVideoDeviceInfo> VideoDeviceInfos { get; set; } = new List<CacheVideoDeviceInfo>();

        /// <summary>
        /// 直播开关，true=开启，false=关闭
        /// </summary>
        /// <value></value>
        public bool LiveOpened { get; set; }

        /// <summary>
        /// 是否支持Rtc
        /// </summary>
        /// <value></value>
        public bool SupportRtc { get; set; }

        /// <summary>
        /// 设备展示名称
        /// </summary>
        /// <value></value>
        public string DisplayName
        {
            get
            {
                var displayName = Description;
                if (string.IsNullOrWhiteSpace(displayName))
                {
                    displayName = Name;
                }
                return displayName;
            }
        }

        /// <summary>
        /// 连接方式
        /// </summary>
        /// <value></value>
        public int ProxyType { get; set; } = 0;
    }

    /// <summary>
    /// 视屏设备画面信息
    /// </summary>
    public class CacheVideoDeviceInfo
    {
        /// <summary>
        /// 设备Id（比如超声机唯一码，用来表示这是超声机画面）
        /// </summary>
        /// <value></value>
        public string VideoDeviceId { get; set; }

        /// <summary>
        /// 视屏画面设备来源的类型（设备主屏或摄像头）
        /// </summary>
        /// <value></value>
        public int VideoDeviceSourceType { get; set; }

        /// <summary>
        /// 设备设置的画面宽度，上报值
        /// </summary>
        /// <value></value>
        public int Width { get; set; }

        /// <summary>
        /// 设备设置的画面高度，上报值
        /// </summary>
        /// <value></value>
        public int Height { get; set; }

        /// <summary>
        /// 设备设置的画面宽度
        /// </summary>
        /// <value></value>
        public int OutputWidth { get; set; }

        /// <summary>
        /// 设备设置的画面高度
        /// </summary>
        /// <value></value>
        public int OutputHeight { get; set; }

        /// <summary>
        /// 帧率
        /// </summary>
        /// <value></value>
        public int VideoFps { get; set; } = 20;

        /// <summary>
        /// 码率
        /// </summary>
        /// <value></value>
        public int VideoBitrate { get; set; } = 2000;

        /// <summary>
        /// 最小码率
        /// </summary>
        /// <value></value>
        public int MinVideoBitrate { get; set; } = 1000;
    }
}