using System;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.OpLog;
namespace WingServerCommon.Interfaces.Cache
{
    public interface IOrganizationsManager : IBaseCacheManager<CacheOrganizationDTO>
    {

    }

    public class OrganizationsManager : CacheManager<CacheOrganizationDTO>, IOrganizationsManager
    {
        public OrganizationsManager() : base()
        {
        }
    }

    public class CacheOrganizationDTO : ICacheObject
    {
        /// <summary>
        /// 组织编码
        /// </summary>
        /// <value>Test00001</value>
        public string Code { get; set; }

        /// <summary>
        /// 组织名称
        /// </summary>
        /// <value>门诊部</value>
        public string OrganizationName { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        /// <value>机构描述</value>
        public string Description { get; set; }

        /// <summary>
        /// 顶级编码
        /// </summary>
        /// <value></value>
        public string RootCode { get; set; }

        /// <summary>
        /// 组织类型
        /// </summary>
        /// <value>0</value>
        public int OrganizationType { get; set; }

        /// <summary>
        /// 所属管理组列表
        /// </summary>
        /// <value>"",""</value>
        public List<string> AuthorityGroups { get; set; } = new();

        /// <summary>
        /// 经纬度
        /// </summary>
        /// <value>38.5</value>
        public string Nautica { get; set; }

        /// <summary>
        /// 审核状态
        /// </summary>
        /// <value>0</value>
        public int State { get; set; }

        /// <summary>
        /// 机构负责人
        /// </summary>
        /// <value>"user001","user002"</value>
        public List<string> Directors { get; set; }

        /// <summary>
        /// 已分配的管理员列表
        /// </summary>
        /// <value>"admin001","admin002"</value>
        public List<string> AssignedAdmins { get; set; }

        /// <summary>
        /// 患者配置
        /// </summary>
        /// <value></value>
        public string PatientSettingJson { get; set; }

        /// <summary>
        /// 检查配置
        /// </summary>
        /// <value></value>
        public string ExamSettingJson { get; set; }

        /// <summary>
        /// 患者配置版本
        /// </summary>
        /// <value></value>
        public string PatientSettingVersion { get; set; }

        /// <summary>
        /// 检查配置版本
        /// </summary>
        /// <value></value>
        public string ExamSettingVersion { get; set; }
        /// <summary>
        /// 最精确地区编码
        /// </summary>
        /// <value>320500</value>
        public string RegionCode { get; set; }

        /// <summary>
        /// 父级编码
        /// </summary>
        /// <value>Test0000501_1</value>
        public string ParentCode { get; set; }

        /// <summary>
        /// 短码
        /// </summary>
        /// <value>TVDSR1</value>
        public string ShortCode { get; set; }

        /// <summary>
        /// logo图片地址
        /// </summary>
        /// <value>http://192.168.6.66:9001/Storage/B1154C6500E4F174249224B41284498E_7.PNG</value>
        public string LogoUrl { get; set; }

        /// <summary>
        /// 检查服务病人对象类型
        /// </summary>
        /// <value>0</value>
        public int PatientType { get; set; }

        /// <summary>
        /// 是否是虚拟机构
        /// </summary>
        /// <value></value>
        public bool Isinvented { get; set; } = false;

        /// <summary>
        /// 配置列表
        /// </summary>
        /// <value></value>
        public List<CacheSettingDTOClass> Settings { get; set; } = new();

        /// <summary>
        /// AI应用集合
        /// </summary>
        public List<string> DiagnosisModules { get; set; } = new();

        /// <summary>
        /// 敏感信息加密显示
        /// </summary>
        /// <value>true</value>
        public bool IsEncryptedShow { get; set; } = true;

        /// <summary>
        /// 可转诊机构
        /// </summary>
        /// <value></value>
        public List<string> ReferralOrganizationCodes { get; set; } = new List<string>();

        /// <summary>
        /// 默认词条库编码
        /// </summary>
        /// <value></value>
        public string DefaultThesaurusCode { get; set; }

        /// <summary>
        /// 默认报告模板编码
        /// </summary>
        /// <value></value>
        public string DefaultReportTemplateCode { get; set; }  

        /// <summary>
        /// 即时会诊是否需要填写病人信息
        /// </summary>
        /// <value></value>
        public bool EmergencyNeedFill { get; set; }

        /// <summary>
        /// 即时会诊配置版本号
        /// </summary>
        /// <value></value>
        public string EmergencySettingVersion { get; set; }

        /// <summary>
        /// 即时会诊病人信息
        /// </summary>
        /// <value></value>
        public string EmergencySettingJson { get; set; }
    }

    public class CacheSettingDTOClass
    {
        /// <summary>
        /// 语言key
        /// </summary>
        /// <value>en-US</value>
        public string LangugeKey { get; set; }

        /// <summary>
        /// 配置模块
        /// </summary>
        /// <value>exam.json</value>
        public string SettingModule { get; set; }

        /// <summary>
        /// 自定义内容Json
        /// </summary>
        /// <value></value>
        public string CustomContent { get; set; }

        /// <summary>
        /// 完整内容（基础配置拼接自定义内容）
        /// </summary>
        /// <value></value>
        public string CompleteContent { get; set; }

        /// <summary>
        /// 基于哪个版本进行自定义的，对比用
        /// </summary>
        /// <value></value>
        public string BaseConfigVersion { get; set; }

        /// <summary>
        /// 语言版本
        /// </summary>
        /// <value></value>
        public string Version { get; set; }
    }
}