namespace WingServerCommon.Interfaces.FileTransfer
{  
    /// <summary>
    /// 分片传输记录实体
    /// </summary>
    public class FileTransferRecorder
    {
        /// <summary>
        /// 原始文件路径
        /// </summary>
        public string FilePath { get; set; }

        /// <summary>
        /// 真实文件名称
        /// </summary>
        public string FileName { get; set; }

        /// <summary>
        /// 上传服务地址
        /// </summary>
        public string Host { get; set; }

        /// <summary>
        /// 文件大小
        /// </summary>
        public long FileSize { get; set; } = 0;

        /// <summary>
        /// 是否上传完成
        /// </summary>
        public bool IsUploadComplete { get; set; } = false;

        /// <summary>
        /// 文件扩展名
        /// </summary>
        public string FileExtension { get; set; }

        /// <summary>
        /// 重命名文件名 
        /// </summary>
        public string ObjectName { get; set; }

        /// <summary>
        /// 是否copy
        /// </summary>
        public bool IsCopy { get; set; }

        /// <summary>
        /// 分块大小
        /// </summary>
        public long SliceSizeForUpload { get; set; }

        /// <summary>
        /// 是否自动分块上传，默认true
        /// </summary>
        public bool IsNeedPartUpload { get; set; } = true;
        
        /// <summary>
        /// 分片记录列表
        /// </summary>
        public List<FileTransferRecorderDetail> PartList { get; set; } = new List<FileTransferRecorderDetail>();
    }

    /// <summary>
    /// 分片传输记录详情实体
    /// </summary>
    public class FileTransferRecorderDetail
    {
        /// <summary>
        /// 分块编号
        /// </summary>
        public int PartNumber { get; set; }

        /// <summary>
        /// 分块名称
        /// </summary>
        public string PartFileName { get; set; }

        /// <summary>
        /// 分块文件大小
        /// </summary>
        public int PartFileSize { get; set; }

        /// <summary>
        /// 分片是否上传完成
        /// </summary>
        public bool IsUploadComplete { get; set; } = false;

        /// <summary>
        /// 自动分片上传初始化响应列表
        /// </summary>
        public InitiateMultipartUploadResult InitiateMultipartUploadResult { get; set; } = new InitiateMultipartUploadResult();
    }

    /// <summary>
    /// 分块上传响应实体
    /// </summary>
    public class InitiateMultipartUploadResult
    {
        /// <summary>
        /// 存储桶
        /// </summary>
        public string Bucket {get; set;}

        /// <summary>
        /// 文件名称 
        /// </summary>
        public string Key {get; set;}

        /// <summary>
        /// uploadId唯一值，分块上传时回传过来
        /// </summary>
        public string UploadId {get; set;}
    }

    /// <summary>
    /// 结束分块上传
    /// </summary>
    public class CompleteMultipartUpload
    {
        /// <summary>
        /// 分块列表
        /// </summary>
        public List<PartInfo> Part {get; set;} = new List<PartInfo>();
    }

    /// <summary>
    /// 分块实体
    /// </summary>
    public class PartInfo
    {
        /// <summary>
        /// 存储分块编码
        /// </summary>
        public int PartNumber {get; set;}

        /// <summary>
        /// 存储tag
        /// </summary>
        public string ETag {get; set;}
    }

    /// <summary>
    /// 完成上传响应实体
    /// </summary>
    public class CompleteMultipartUploadResult
    {
        /// <summary>
        /// 存储地址url
        /// </summary>
        public string Location {get; set;}

        /// <summary>
        /// 存储桶
        /// </summary>
        public string Bucket {get; set;}

        /// <summary>
        /// 文件地址
        /// </summary>
        public string Key {get; set;}

        /// <summary>
        /// ETag
        /// </summary>
        public string ETag {get; set;}
    }
}