using JsonRpcLite.Services;

namespace WingServerCommon.Service
{
    public class JsonRpcService : IJsonRpcService
    {
        private bool _disposed;
        private JsonRpcClientPool? _jsonRpcClientPool;

        /// <summary>
        /// Release resource when disposing.
        /// </summary>
        protected virtual void Disposing()
        {
            _jsonRpcClientPool = null;
        }

        protected void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                Disposing();
                _disposed = true;
            }
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        public virtual void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            _jsonRpcClientPool = jsonRpcClientPool;
        }

        protected void ThrowRpcException(int code, string message, string internalMessage = null)
        {
            throw new RpcException(code, message, internalMessage ?? message);
        }

        protected T GetProxy<T>()
        {
            var client = _jsonRpcClientPool?.GetJsonRpcClient<T>();
            if (client != null)
            {
                var proxy = client.CreateProxy<T>();
                if (proxy == null)
                {
                    throw new NotSupportedException($"Can not get proxy with type{typeof(T)} in rpc client");
                }

                return proxy;
            }

            throw new NotSupportedException($"Can not get proxy with type{typeof(T)}");
        }

        public virtual void LoadAfterRegister()
        {

        }
    }
}