using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using WingServerCommon.Log;

namespace WingServerCommon.Utilities
{
    /// <summary>
    /// Base64 加密、解密
    /// <para>主要方法如下：</para>
    /// <para>01. Encrypt(string text)  //加密字符串</para>
    /// <para>02. Decrypt(string text)  //解密字符串</para>
    /// 6Bit数字【0~63】映射Base64字符表如下,补位我们使用=等号代替，按(64,'=')处理
    /// </summary>
    public class CryptionHelper
    {

        public static string EncryptOneByOne(string text)
        {
            var result = "";
            for (int i = 0; i < text.Length; i++)
            {
                result += Encrypt(text[i].ToString());
            }
            return result;
        }

        /// <summary>
        /// Base64加密
        /// </summary>
        /// <param name="text">待加密的字符串</param>
        /// <returns>加密后字符串</returns>
        public static string Encrypt(string text)
        {
            //如果字符串为空，则返回
            if (string.IsNullOrEmpty(text))
            {
                return "";
            }

            try
            {
                char[] Base64Code = new char[]{'0','1','2','3','4','5','6','7',
                                            '8','9','+','/','a','b','c','d','e','f','g','h','i','j','k','l','m','n',
                                            'o','p','q','r','s','t','u','v','w','x','y','z','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T',
                                            'U','V','W','X','Y','Z','='};
                byte empty = (byte)0;
                ArrayList byteMessage = new ArrayList(Encoding.UTF8.GetBytes(text));
                StringBuilder outmessage;
                int messageLen = byteMessage.Count;
                int page = messageLen / 3;
                int use = 0;
                if ((use = messageLen % 3) > 0)
                {
                    for (int i = 0; i < 3 - use; i++)
                        byteMessage.Add(empty);
                    page++;
                }
                outmessage = new System.Text.StringBuilder(page * 4);
                for (int i = 0; i < page; i++)
                {
                    byte[] instr = new byte[3];
                    instr[0] = (byte)byteMessage[i * 3];
                    instr[1] = (byte)byteMessage[i * 3 + 1];
                    instr[2] = (byte)byteMessage[i * 3 + 2];
                    int[] outstr = new int[4];
                    outstr[0] = instr[0] >> 2;
                    outstr[1] = ((instr[0] & 0x03) << 4) ^ (instr[1] >> 4);
                    if (!instr[1].Equals(empty))
                        outstr[2] = ((instr[1] & 0x0f) << 2) ^ (instr[2] >> 6);
                    else
                        outstr[2] = 64;
                    if (!instr[2].Equals(empty))
                        outstr[3] = (instr[2] & 0x3f);
                    else
                        outstr[3] = 64;
                    outmessage.Append(Base64Code[outstr[0]]);
                    outmessage.Append(Base64Code[outstr[1]]);
                    outmessage.Append(Base64Code[outstr[2]]);
                    outmessage.Append(Base64Code[outstr[3]]);
                }
                var result = outmessage.ToString();
                return result;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }



        /// <summary>
        /// Base64解密
        /// </summary>
        /// <param name="text">待解密的字符串</param>
        /// <returns>解密后的字符串</returns>
        public static string Decrypt(string text)
        {
            //如果字符串为空，则返回
            if (string.IsNullOrEmpty(text))
            {
                return "";
            }

            //将空格替换为加号
            text = text.Replace(" ", "+");

            try
            {
                if ((text.Length % 4) != 0)
                {
                    Logger.WriteLineWarn($"包含不正确的BASE64编码：" + text);
                    return "";
                }
                if (!Regex.IsMatch(text, "^[A-Z0-9/+=]*$", RegexOptions.IgnoreCase))
                {

                    Logger.WriteLineWarn($"包含不正确的BASE64编码：" + text);
                    return "";
                }
                string Base64Code = "0123456789+/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ=";
                int page = text.Length / 4;
                ArrayList outMessage = new ArrayList(page * 3);
                char[] message = text.ToCharArray();
                for (int i = 0; i < page; i++)
                {
                    byte[] instr = new byte[4];
                    instr[0] = (byte)Base64Code.IndexOf(message[i * 4]);
                    instr[1] = (byte)Base64Code.IndexOf(message[i * 4 + 1]);
                    instr[2] = (byte)Base64Code.IndexOf(message[i * 4 + 2]);
                    instr[3] = (byte)Base64Code.IndexOf(message[i * 4 + 3]);
                    byte[] outstr = new byte[3];
                    outstr[0] = (byte)((instr[0] << 2) ^ ((instr[1] & 0x30) >> 4));
                    if (instr[2] != 64)
                    {
                        outstr[1] = (byte)((instr[1] << 4) ^ ((instr[2] & 0x3c) >> 2));
                    }
                    else
                    {
                        outstr[2] = 0;
                    }
                    if (instr[3] != 64)
                    {
                        outstr[2] = (byte)((instr[2] << 6) ^ instr[3]);
                    }
                    else
                    {
                        outstr[2] = 0;
                    }
                    outMessage.Add(outstr[0]);
                    if (outstr[1] != 0)
                        outMessage.Add(outstr[1]);
                    if (outstr[2] != 0)
                        outMessage.Add(outstr[2]);
                }
                byte[] outbyte = (byte[])outMessage.ToArray(Type.GetType("System.Byte"));
                var result = Encoding.UTF8.GetString(outbyte);
                if (!IsNotErrorCode(result))
                {
                    Logger.WriteLineWarn($"包含不正确的BASE64编码：" + text);
                    return "";
                }
                return result;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 手机号加密
        /// </summary>
        /// <param name="phone"></param>
        /// <returns></returns>
        public static string PhoneEncrypt(string phone, bool isEncryptedShow = true)
        {
            if (string.IsNullOrWhiteSpace(phone))
            {
                return phone;
            }
            var tempPhone = Decrypt(phone);
            if (isEncryptedShow)
            {
                tempPhone = ReplaceWithSpecialChar(tempPhone, 3, 4);
            }
            return tempPhone;
        }

        /// <summary>
        /// 身份证加密
        /// </summary>
        /// <param name="idNumber"></param>
        /// <returns></returns>
        public static string IDNumberEncrypt(string idNumber, bool isEncryptedShow = true)
        {
            if (string.IsNullOrWhiteSpace(idNumber))
            {
                return idNumber;
            }
            var tempID = Decrypt(idNumber);
            if (isEncryptedShow)
            {
                tempID = ReplaceWithSpecialChar(tempID, 3, 3);
            }
            return tempID;
        }

        /// <summary>
        ///社保编号加密
        /// </summary>
        /// <param name="ssNumber"></param>
        /// <returns></returns>
        public static string SSNumberEncrypt(string ssNumber, bool isEncryptedShow = true)
        {
            if (string.IsNullOrWhiteSpace(ssNumber))
            {
                return ssNumber;
            }
            var tempID = Decrypt(ssNumber);
            if (isEncryptedShow)
            {
                tempID = ReplaceWithSpecialChar(tempID, 5, 0);
            }
            return tempID;
        }

        public static bool IsChinese(char c)
        {
            return 0x4e00 <= c && c <= 0x9fbb;
        }

        public static bool IsChinese(string str)
        {
            char[] ch = str.ToCharArray();
            for (int i = 0; i < ch.Length; ++i)
            {
                if (IsChinese(ch[i]))
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// 是否正常字符串不含乱码
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool IsNotErrorCode(string str)
        {
            char[] ch = str.ToCharArray();
            for (int i = 0; i < ch.Length; ++i)
            {
                 //乱码
                 if(ch[i]<7||(ch[i]>13&&ch[i]<26)||(ch[i]>27&&ch[i]<32)||ch[i]>65532)
                 {
                    return false;
                 }
            }
            return true;
        }

        /// <summary>
        ///姓名加密
        /// </summary>
        /// <param name="ssNumber"></param>
        /// <returns></returns>
        public static string NameEncrypt(string name, bool isEncryptedShow = true)
        {
            if (string.IsNullOrWhiteSpace(name))
            {
                return name;
            }
            name = Decrypt(name);
            if (!isEncryptedShow)
            {
                return name;
            }
            if (IsChinese(name))
            {

                //中文
                if (name.Length == 2)
                {
                    name = name.Substring(0, 1) + "*";
                }
                else if (name.Length > 2)
                {
                    name = ReplaceWithSpecialChar(name, 1, 1);
                }
            }
            else//外文
            {
                var wordList = name.Split(' ');
                if (wordList.Length == 1)
                {
                    if (name.Length == 2)
                    {
                        name = name.Substring(0, 1) + " *";
                    }
                    else if (name.Length > 2)
                    {
                        name = ReplaceWithSpecialChar(name, 1, 1);
                    }
                }
                else if (wordList.Length == 2)
                {
                    name = "**** " + wordList[1];
                }
                else
                {
                    name = wordList[0] + " **** " + wordList[wordList.Length - 1];
                }
            }
            return name;
        }

        /// <summary>
        /// 将传入的字符串中间部分字符替换成特殊字符
        /// </summary>
        /// <param name="value">需要替换的字符串</param>
        /// <param name="startLen">前保留长度</param>
        /// <param name="endLen">尾保留长度</param>
        /// <param name="replaceChar">特殊字符</param>
        /// <returns>被特殊字符替换的字符串</returns>
        private static string ReplaceWithSpecialChar(string value, int startLen = 4, int endLen = 4, char specialChar = '*')
        {
            try
            {
                if (string.IsNullOrWhiteSpace(value))
                {
                    return value;
                }
                int lenth = value.Length - startLen - endLen;
                if (lenth <= 0)
                {
                    return value.Substring(0, value.Length - 1) + "*";
                }
                var atartPart = value.Substring(0, startLen);
                var middlePart = "";
                for (int i = 0; i < lenth; i++)
                {
                    middlePart += specialChar;
                }
                var endPart = value.Substring(value.Length - endLen, endLen);
                value = atartPart + middlePart + endPart;
            }
            catch (Exception)
            {
                throw;
            }
            return value;
        }

    }


}