﻿using System;
using System.Collections.Generic;

namespace WingServerCommon.Utilities.Executors
{
    class StackExecutor<T> : ExecutorBase<T> where T : class
    {
        private readonly Stack<ExecuteWorker<T>> _workers = new Stack<ExecuteWorker<T>>();

        public StackExecutor(string name) : base(name)
        {
        }

        protected StackExecutor(string name, TimeSpan executeDelayTime) : base(name, executeDelayTime)
        {
        }

        /// <summary>
        /// Gets current worker count in stack.
        /// </summary>
        /// <returns></returns>
        protected override int GetWorkerCount()
        {
            return _workers.Count;
        }

        /// <summary>
        /// Add one worker into the stack.
        /// </summary>
        /// <param name="worker"></param>
        protected override void AddWorker(ExecuteWorker<T> worker)
        {
            _workers.Push(worker);
        }

        /// <summary>
        /// Take one worker from the stack.
        /// </summary>
        /// <returns></returns>
        protected override ExecuteWorker<T> TakeWorker()
        {
            return _workers.Pop();
        }

        /// <summary>
        /// Gets all workers.
        /// </summary>
        /// <returns></returns>
        protected override IEnumerable<ExecuteWorker<T>> GetWorkers()
        {
            return _workers.ToArray();
        }

        /// <summary>
        /// Clear all workers which not stated.
        /// </summary>
        protected override void CancelWorkers()
        {
            _workers.Clear();
        }
    }
}
